<?php

/**
 * @file
 * Provides admin forms and functions for commerce_stripe.
 */

use Stripe\PaymentIntent;
use Stripe\Refund;

/**
 * Form callback for processing refunds.
 */
function commerce_stripe_ideal_refund_form($form, &$form_state, $order, $transaction) {

  $form = array();
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $order_wrapper = entity_metadata_wrapper('commerce_order',$order);
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;
  $form_state['payment_method'] = $payment_method;
  $form_state['balance'] = commerce_payment_order_balance($form_state['order']);
  $form_state['remaining'] = $order_wrapper->commerce_order_total->amount->value() - $form_state['balance']['amount'];

  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Refund amount'),
    '#description' => t('Enter any amount to refund up to @txn_amount', array('@txn_amount' => commerce_currency_format($form_state['remaining'], $transaction->currency_code))),
    '#required' => TRUE,
    '#size' => 8,
  );

  $options = array(
    'requested_by_customer' => t('Requested by customer'),
    'duplicate' => t('Duplicate'),
    'fraudulent' => t('Fraudulent'),
  );

  $form['reason'] = array(
    '#type' => 'select',
    '#title' => t('Refund reason'),
    '#description' => t('Select the most appropriate reason for the refund.'),
    '#options' => $options,
  );

  $form['actions'] = array(
    '#type' => 'container',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Process refund'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'button',
    '#value' => t('Cancel'),
  );
  return $form;
}

/**
 * Validation callback for submitting refunds to Stripe.
 */
function commerce_stripe_ideal_refund_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
  // Make sure the refund amount is valid and available.
  if ($form_state['values']['amount'] <= 0 || $form_state['values']['amount'] > commerce_currency_amount_to_decimal($form_state['remaining'], $transaction->currency_code) || !is_numeric($form_state['values']['amount'])) {
    form_set_error('amount', t('Please enter a valid return amount that is less than or equal to the remaining balance available for refund of @remaining.', array('@remaining' => commerce_currency_format($form_state['remaining'], $transaction->currency_code))));
  }
}

/**
 * Submit callback for submitting refunds to Stripe.
 */
function commerce_stripe_ideal_refund_form_submit($form, &$form_state) {
  $order = $form_state['order'];
  $transaction = $form_state['transaction'];
  $payment_method = $form_state['payment_method'];

  // Make sure the library is available.
  if (!commerce_stripe_ideal_load_library($form_state['payment_method']['settings']['secret_key'])) {
    drupal_set_message(t('Cannot load the Stripe PHP library'), 'error');
    return FALSE;
  }

  try {
    $payment_intent = PaymentIntent::retrieve($transaction->remote_id);
    // Create the refund object.
    $data = array(
      'charge' => reset($payment_intent->charges->data),
      'amount' => commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code),
      'reason' => $form_state['values']['reason'],
    );
    $refund = Refund::create($data);
    if ($refund instanceof Refund) {
      // Create the new commerce payment transaction and set appropriate values.
      $refund_transaction = commerce_payment_transaction_new($transaction->payment_method, $order->order_id);
      $refund_transaction->instance_id = $payment_method['instance_id'];
      $refund_transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $refund_transaction->remote_id = $refund->id;
      $refund_transaction->message = t('Refund issued.');
      $refund_transaction->payload = $refund->toArray();
      $refund_transaction->remote_status = $refund->status; // Create function returns the Refund object if the refund succeeded otherwise throws exception (from Stripe API)
      // Save the amount as a negative integer.
      $refund_transaction->amount = $refund->amount * -1;
      $refund_transaction->currency_code = strtoupper($payment_intent->currency);

      commerce_payment_transaction_save($refund_transaction);

      // Inform the user of the success and redirect them back to payments.
      drupal_set_message(t('Refund processed successfully'));
      $form_state['redirect'] = 'admin/commerce/orders/' . $order->order_id . '/payment';
    }
  }
  catch (Exception $e) {
    drupal_set_message(t('The transaction could not be refunded. The error was: @error',
      array('@error' => $e->getMessage())), 'error');
  }
}

/**
 * Form callback for processing refunds.
 */
function commerce_stripe_ideal_void_form($form, &$form_state, $order, $transaction) {

  $form = array();
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $order_wrapper = entity_metadata_wrapper('commerce_order',$order);
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;
  $form_state['payment_method'] = $payment_method;
  $form_state['balance'] = commerce_payment_order_balance($form_state['order']);
  $form_state['remaining'] = $order_wrapper->commerce_order_total->amount->value() - $form_state['balance']['amount'];

  $form['description'] = array(
    '#type' => 'markup',
    '#markup' => t('<p>Note: Once canceled, no additional charges will be made by the Payment Intent and any operations on the Payment Intent will fail with an error. For Payment Intents with status=\'requires_capture\', the remaining amount_capturable will automatically be refunded.</p>'),
  );

  $options = array(
    'requested_by_customer' => t('Requested by customer'),
    'duplicate' => t('Duplicate'),
    'Fraudulent' => t('Fraudulent'),
    'abandoned' => t('Abandoned'),
  );

  $form['reason'] = array(
    '#type' => 'select',
    '#title' => t('Void reason'),
    '#description' => t('Select the most appropriate reason for the void.'),
    '#options' => $options,
  );

  $form['actions'] = array(
    '#type' => 'container',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Void'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );
  return $form;
}

/**
 * Validation callback for submitting refunds to Stripe.
 */
function commerce_stripe_ideal_void_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
}

/**
 * Submit callback for submitting refunds to Stripe.
 */
function commerce_stripe_ideal_void_form_submit($form, &$form_state) {
  // if triggering element was cancel to to payments
  if ($form_state['triggering_element']['#parents'][0] === 'cancel') {
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
  $order = $form_state['order'];
  $transaction = $form_state['transaction'];
  commerce_stripe_ideal_void($transaction, $order, $form_state['values']['reason']);
  $form_state['redirect'] = 'admin/commerce/orders/' . $order->order_id . '/payment';
}

/**
 * Helper function for void some custom void payment implementations
 *
 * @param bool|object $order
 * @param $transaction
 * @param $reason
 * @return bool
 */
function commerce_stripe_ideal_void($transaction, $order = FALSE, $reason = FALSE) {
  $order = is_object($order) ? $order : commerce_order_load($transaction->order_id);
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  // Make sure the library is available.
  if (!commerce_stripe_ideal_load_library($payment_method['settings']['secret_key'])) {
    drupal_set_message(t('Cannot load the Stripe PHP library'), 'error');
    return FALSE;
  }

  try {
    $payment_intent = PaymentIntent::retrieve($transaction->remote_id);
    $params = $reason ? array('cancellation_reason'=> $reason) : array();
    $payment_intent->cancel($params);
    if ($payment_intent instanceof PaymentIntent && $payment_intent->status == 'canceled') {
      // Create the new commerce payment transaction and set appropriate values.
      $transaction = commerce_payment_transaction_new($transaction->payment_method, $order->order_id);
      $transaction->instance_id = $payment_method['instance_id'];
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->remote_id = $payment_intent->id;
      $transaction->message = t('Void processed successfully.');
      $transaction->payload = $payment_intent->toArray();
      $transaction->remote_status = $payment_intent->status;
      // Save the amount as a negative integer.
      $transaction->amount = $payment_intent->amount * -1;
      $transaction->currency_code = strtoupper($payment_intent->currency);
      commerce_payment_transaction_save($transaction);

      // Inform the user of the success and redirect them back to payments.
      drupal_set_message(t('Void processed successfully'));
      return TRUE;
    }
  }
  catch (Exception $e) {
    drupal_set_message(t('The transaction could not be voided. The error was: @error',
      array('@error' => $e->getMessage())), 'error');
  }
  return FALSE;
}

