<?php

/**
* @file
* Provides webhook functionality for stripe.
*/


function commerce_stripe_ideal_webhook(){
  $payment_method = commerce_payment_method_instance_load('commerce_stripe_ideal|commerce_payment_commerce_stripe_ideal');
  commerce_stripe_ideal_load_library($payment_method['settings']['secret_key']);

  // If you are testing your webhook locally with the Stripe CLI you
  // can find the endpoint's secret by running `stripe listen`
  // Otherwise, find your endpoint's secret in your webhook settings in the Developer Dashboard
  $endpoint_secret = $payment_method['settings']['signing_secret'];

  $payload = @file_get_contents('php://input');
  $sig_header = isset($_SERVER['HTTP_STRIPE_SIGNATURE']) ? $_SERVER['HTTP_STRIPE_SIGNATURE'] : FALSE;
  $event = null;

  try {
    $event = \Stripe\Webhook::constructEvent(
      $payload, $sig_header, $endpoint_secret
    );
  } catch(\UnexpectedValueException $e) {
    watchdog('commerce_stripe_ideal', $e->getMessage());
    // Invalid payload
    http_response_code(400);
    exit();
  } catch(\Stripe\Exception\SignatureVerificationException $e) {
    watchdog('commerce_stripe_ideal', $e->getMessage());
    // Invalid signature
    http_response_code(400);
    exit();
  }
  watchdog('commerce_stripe_ideal', print_r($event->toArray(), TRUE));

  // Handle the event
  switch ($event->type) {
    case 'payment_intent.succeeded':
      $paymentIntent = $event->data->object; // contains a StripePaymentIntent
      commerce_stripe_ideal_handle_payment_intent_succeeded($paymentIntent,t('Transaction updated via webhook. Moved to success'));
      break;
    case 'payment_intent.payment_failed':
    case 'payment_intent.canceled':
      $paymentIntent = $event->data->object; // contains a StripePaymentIntent
      commerce_stripe_ideal_handle_payment_intent_failed($paymentIntent, t('Transaction updated via webhook. Moved to canceled'));
      break;
    default:
      // We don't support other events so return error code
      http_response_code(400);
      exit();
  }

  http_response_code(200);
}

/**
 * Handles payment intent succeeded event.
 *
 * @param \Stripe\PaymentIntent $payment_intent
 *   Payment intent.
 * @param string $message
 *   Message for transaction.
 */
function commerce_stripe_ideal_handle_payment_intent_succeeded($payment_intent, $message){
  $order_id = $payment_intent->metadata->offsetGet('order_id');
  $order = $order_id ? commerce_order_load($order_id) : FALSE;
  //Handle this only in case payment intent matches the one on order otherwise do nothing
  if (is_object($order) &&
    isset($order->data['commerce_stripe_ideal']) &&
    $order->data['commerce_stripe_ideal']['payment_intent'] == $payment_intent->id
    && isset($order->data['commerce_stripe_ideal']['payment_method_instance'])) {
    $payment_method = commerce_payment_method_instance_load($order->data['commerce_stripe_ideal']['payment_method_instance']);
    commerce_stripe_ideal_transaction($payment_method, $order, $payment_intent, COMMERCE_PAYMENT_STATUS_SUCCESS, $message);
    commerce_checkout_complete($order);
    foreach (module_implements('commerce_stripe_ideal_payment_intent_succeeded') as $module) {
      $function = $module . '_commerce_stripe_ideal_payment_intent_succeeded';
      $function($payment_intent, $order);
    }
  }
}

/**
 * Handles payment intent succeeded event.
 *
 * @param \Stripe\PaymentIntent $payment_intent
 *   Payment intent.
 * @param string $message
 *   Message for transaction.
 */
function commerce_stripe_ideal_handle_payment_intent_failed($payment_intent, $message){
  $order_id = $payment_intent->metadata->offsetGet('order_id');
  $order = $order_id ? commerce_order_load($order_id) : FALSE;
  //Handle this only in case payment intent matches the one on order otherwise do nothing
  if (is_object($order) &&
    isset($order->data['commerce_stripe_ideal']) &&
    $order->data['commerce_stripe_ideal']['payment_intent'] == $payment_intent->id
    && isset($order->data['commerce_stripe_ideal']['payment_method_instance'])) {
    If($payment_intent->status == \Stripe\PaymentIntent::STATUS_REQUIRES_PAYMENT_METHOD){
      //ideal payment was not successfull (3d secure or bank failed) so we can cancel it so it doesn't left hanging.
      $payment_intent->cancel();
    }
    $payment_method = commerce_payment_method_instance_load($order->data['commerce_stripe_ideal']['payment_method_instance']);
    commerce_stripe_ideal_transaction($payment_method, $order, $payment_intent, COMMERCE_PAYMENT_STATUS_FAILURE, $message);
    commerce_order_status_update($order,'canceled');
    foreach (module_implements('commerce_stripe_ideal_payment_intent_failed') as $module) {
      $function = $module . '_commerce_stripe_ideal_payment_intent_failed';
      $function($payment_intent, $order);
    }
  }
}
